/*   B_WIN.LIB
     B_WIN.C
     This is a small C++ windowing library that was developed with TC++
     This library is now PUBLIC DOMAIN, and may be freely copied, or
     modified to your hearts content.  It would be nice though to send me
     a copy of your modifications.
     Robert M. Place II
     Quarters 384 D.
     Quantico, Va. 22134
*/

/*************************************************************************
INCLUDE SECTION
*************************************************************************/
     #include <iostream.h>
     #include <stdio.h>
     #include <stdlib.h>
     #include <dos.h>
     #include <conio.h>
     #include <string.h>
     #include <bios.h>

     /****************************************************************
     DEFINES SECTION
     *****************************************************************/

     // SYSTEM DEFINES
     #define CURSIZE    1         /* used with cursor functions           */
     #define VIDEO      0x10      /* video inturupt                       */
     #define STOPBIT    0x20      /* used with cursor functions           */
     #define MOUSE      0x33      /* mouse inturupt                       */
     #define TRUE       1

     // BOARDER COLORS  These can have the bright attribute added.
     #define   BLACK          0
     #define   BLUE           1
     #define   GREEN          2
     #define   CYAN           3
     #define   RED            4
     #define   MAGENTA        5
     #define   BROWN          6
     #define   LT_GRAY        7

     // TEXT FORGROUND COLORS
     #define   DK_GRAY        8
     #define   LT_BLUE        9
     #define   LT_GREEN       10
     #define   LT_CYAN        11
     #define   LT_RED         12
     #define   LT_MAGENTA     13
     #define   YELLOW         14
     #define   WHITE          15

     // COLOR ATTRIBUTES
     #define   YES_BLINK          1
     #define   NO_BLINK       0
     #define   BRIGHT         1
     #define   NORMAL         0

     // BOARDER ATTRIBUTES
     #define   SHADOW         1
     #define   NO_SHADOW      0

     //WINDOW CONSTANTS
     #define   MAX_WINDOWS    8 // max number of open windows.

     /****************************************************************
     GLOBAL VARIABLES
     *****************************************************************/

     // MAIN WINDOW STRUCTURE
     struct    win_data_ovl{
          int  top;
          int  left;
          int  bottom;
          int  right;
          int  border_back_color;
          int  border_fore_color;
          int  bright_attrib;
          int  blink_attrib;
          int  text_back_color;
          int  text_color;
          int  win_shadow;
          int  win_border_type;
          int  *under_the_window;
          char *win_title;
          int  cursor_x;
          int  cursor_y;          /* Stores the cursor location in a window */
          int  *win_contents;
     };

     struct get_list_ovl
     {
          int x;
          int y;
          char *string;
          int size;
          char filler;
     };
     struct get_list_ovl get_list[20];

     int  MONITOR_TYPE;
     int  STRING_ABORT;  // used to abort the get_str() function.
     int  CUR_WIN = 0;
     int  get_counter=0;
     int  new_counter=0;
     int  max_get_counter=0;
     struct win_data_ovl win_data[MAX_WINDOWS];



// FUNCTION PROTOTYPES //
int GetAt(int x, int y, char *string, int size, char filler);
int ReadGets(void);
int off(void);
int on(void);
int FillBox(unsigned char,int ,int ,int, int,unsigned int,unsigned int ,
            unsigned int,unsigned int );
int FillItUp(unsigned char, int, int, int, int, int);
int GetVideoMode(void);
int MakeWindow(int, int, int, int, int,int, int , int , int , int , char *,
               int, int);
int BWindow(void);
int CloseWindow(void);
int ColorWindowBorder(int,int,int,int);
int ColorWindowText(int, int);
int ChangeWindowBorder(int );
int WindowShadowOn(void);
int WindowShadowOff(void);
int ChangeWindowTitle(char *);
int get_contents(void);
int replace_contents(void);
int ClearWindow(void);
int MoveWindow(int, int);
int GetStr(char *, int, char);
int cursor_right(void);
int WText(char *, int, int);
int CGetStr(char *, int, char,int ,int);
int GetGetStr(char *,int ,char );


/*************************************************************************
CURSOR FUNCTIONS   Turns the cursor on or off
*************************************************************************/

int off()
{
     union REGS regs;
     regs.h.ch = STOPBIT;
     regs.h.ah = CURSIZE;
     int86(VIDEO, &regs, &regs);
     return(0);
}

int On()
{
     union REGS regs;
     int start=7;
     int end = 8;
     regs.h.ch = (char)start;
     regs.h.cl = (char)end;
     regs.h.ah = CURSIZE;
     int86(VIDEO, &regs, &regs);
     return(0);
}

/*************************************************************************
SCREEN FILL FUNCTIONS  These will fill an area of the screen with characters
*************************************************************************/

int FillBox(unsigned char filling, int left, int top, int right, int bottom,
         unsigned int fg,unsigned int bg,unsigned int brightly,
         unsigned int blinker)
{
     unsigned int temp_attr;
     struct s_type
     {

          unsigned int foreground:3;
          unsigned int intense : 1;
          unsigned int background : 3;
          unsigned int blinker : 1;
     };
     union two_types{
          s_type type_1;
          int  type_2;
     };
     two_types t;
     t.type_1.foreground=fg;
     t.type_1.background=bg;
     t.type_1.intense=brightly;
     t.type_1.blinker=blinker;
     FillItUp(filling,left,top,right,bottom,t.type_2);
     return(0);
}

int FillItUp(unsigned char filling, int left, int top, int right, int bottom,
           int attr)
{
     int far *farptr;
     int address, address1;
     if(MONITOR_TYPE)
          farptr=(int far *)0xb8000000;
     else
          farptr=(int far *)0xb0000000;
     for(address=top-1;address<bottom;address++)
          for(address1=left-1;address1<right;address1++)
              *(farptr+address*80+address1)= filling|attr<<8;
     return(0);
}


/*************************************************************************
VIDEO FUNCTIONS  This function checks the monitor type
*************************************************************************/

 int GetVideoMode()
{
     unsigned int video_mode,data;
     data=biosequip();          /* get the equipment list                */
     video_mode=(data>>4)&3;    /* checks the video bit for monitor type */
     switch(video_mode)
     {
          case 1:MONITOR_TYPE=1;break; /* color monitor                    */
          case 2:MONITOR_TYPE=1;break; /* color monitor                    */
          case 3:MONITOR_TYPE=0;break; /* monochrome monitor               */
     }
     return(0);
}


/*************************************************************************
WINDOW FUNCTIONS  These functions will create and manipulate the windows
*************************************************************************/

int MakeWindow(int left, int top, int right, int bottom, int border_back_color,
           int border_fore_color, int bright_attrib, int blink_attrib,
           int text_back_color, int text_color, char *win_title,
           int win_shadow,
           int win_border_type)
{
     int buf;

////// Save the contents of earlier windows /////
     if(CUR_WIN>0)
     {
          buf=(win_data[CUR_WIN].bottom-win_data[CUR_WIN].top+2)*
              (win_data[CUR_WIN].right-win_data[CUR_WIN].left+2)*2;
          win_data[CUR_WIN].win_contents=(int *)malloc(buf);
          gettext(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
                  win_data[CUR_WIN].right,win_data[CUR_WIN].bottom,
                  win_data[CUR_WIN].win_contents);
          win_data[CUR_WIN].cursor_x=wherex();
          win_data[CUR_WIN].cursor_y=wherey();
     }

//////  IF THIS IS THE FIRST TIME CALLING THE FUNCTION //////
     else
     {
          win_data[CUR_WIN].top=1;
          win_data[CUR_WIN].left=1;
          win_data[CUR_WIN].right=80;
          win_data[CUR_WIN].bottom=25;
          buf=(win_data[CUR_WIN].bottom-win_data[CUR_WIN].top+2)*
              (win_data[CUR_WIN].right-win_data[CUR_WIN].left+2)*2;
          win_data[CUR_WIN].win_contents=(int *)malloc(buf);
          gettext(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
                  win_data[CUR_WIN].right,win_data[CUR_WIN].bottom,
                  win_data[CUR_WIN].win_contents);
          win_data[CUR_WIN].cursor_x=wherex();
          win_data[CUR_WIN].cursor_y=wherey();
     }

     CUR_WIN++;

////// SAVE THE SCREEN UNDER THE NEW WINDOW //////
     buf=(bottom-top+2)*(right-left+2)*2;
     win_data[CUR_WIN].under_the_window=(int *)malloc(buf);
     gettext(left,top,right+1,bottom+1,
             win_data[CUR_WIN].under_the_window);

////// UPDATE THE WINDOW INFORMATION ///////
     win_data[CUR_WIN].top=top;
     win_data[CUR_WIN].bottom=bottom;
     win_data[CUR_WIN].left=left;
     win_data[CUR_WIN].right=right;
     win_data[CUR_WIN].border_back_color=border_back_color;
     win_data[CUR_WIN].border_fore_color=border_fore_color;
     win_data[CUR_WIN].bright_attrib=bright_attrib;
     win_data[CUR_WIN].blink_attrib=blink_attrib;
     win_data[CUR_WIN].text_back_color=text_back_color;
     win_data[CUR_WIN].text_color=text_color;
     win_data[CUR_WIN].win_title=win_title;
     win_data[CUR_WIN].win_shadow=win_shadow;
     win_data[CUR_WIN].win_border_type=win_border_type;
     window(1,1,80,25);
     BWindow();
     return(0);
}


// This function will create or change all of the windows attributes.  It
// will be called from several of the window functions.
int BWindow()
{
     int buf;
     char *shadow_box;
     int x,y;
     int ht=win_data[CUR_WIN].bottom-win_data[CUR_WIN].top+1;
     int wd=win_data[CUR_WIN].right-win_data[CUR_WIN].left+1;
     char ltcorner,rtcorner,lbcorner,rbcorner,lrmarker,udmarker;
     int length=0;
     window(1,1,80,25);
     buf=(win_data[CUR_WIN].bottom-win_data[CUR_WIN].top+2)*
         (win_data[CUR_WIN].right-win_data[CUR_WIN].left+2)*2;
     shadow_box=(char *)malloc(buf);
     gettext(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
             win_data[CUR_WIN].right+1,win_data[CUR_WIN].bottom+1,shadow_box);

////// MAKE A SHADOW FOR THE WINDOW //////
     if((win_data[CUR_WIN].right>78)||
       (win_data[CUR_WIN].win_shadow == NO_SHADOW)||
       (win_data[CUR_WIN].bottom>23))
          win_data[CUR_WIN].win_shadow=NO_SHADOW;
     if(win_data[CUR_WIN].win_shadow==SHADOW)
     {
          textcolor(8);
          textbackground(0);
          window(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
                 win_data[CUR_WIN].right+1,win_data[CUR_WIN].bottom+2);
          for(y=2;y<ht+1;y++)
          {
               gotoxy(wd+1,y);
               putch(*(shadow_box+((wd+1)*y-1)*2));
          }
          gotoxy(2,ht+1);
          for(x=0;x<wd;x++)
               putch(*(shadow_box+(((wd+1)*ht+1)+x)*2));
          textcolor(win_data[CUR_WIN].text_color);
          textbackground(win_data[CUR_WIN].text_back_color);
     }
     free(shadow_box);
     window(1,1,80,25);
     length=strlen(win_data[CUR_WIN].win_title);

////// SET THE BORDER TYPE //////
     switch(win_data[CUR_WIN].win_border_type)
     {
          case 0:ltcorner=' ';
               rtcorner=' ';
               lbcorner=' ';
               rbcorner=' ';
               lrmarker=' ';
               udmarker=' ';
               break;
          case 1:ltcorner=218;
               rtcorner=191;
               lbcorner=192;
               rbcorner=217;
               lrmarker=196;
               udmarker=179;
               break;
          case 2:ltcorner=201;
               rtcorner=187;
               lbcorner=200;
               rbcorner=188;
               lrmarker=205;
               udmarker=186;
               break;
          case 3:ltcorner=213;
               rtcorner=184;
               lbcorner=212;
               rbcorner=190;
               lrmarker=205;
               udmarker=179;
               break;
          case 4:ltcorner=214;
               rtcorner=183;
               lbcorner=211;
               rbcorner=189;
               lrmarker=196;
               udmarker=186;
               break;
     }

///// CREATE THE BOARDER WITH SCREEN WRITES FOR SPEED /////
     FillBox(lrmarker,win_data[CUR_WIN].left,win_data[CUR_WIN].top,
              win_data[CUR_WIN].right,
              win_data[CUR_WIN].top, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(lrmarker,win_data[CUR_WIN].left,win_data[CUR_WIN].bottom,
              win_data[CUR_WIN].right,
              win_data[CUR_WIN].bottom, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(udmarker,win_data[CUR_WIN].left,win_data[CUR_WIN].top,
              win_data[CUR_WIN].left,
              win_data[CUR_WIN].bottom, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(udmarker,win_data[CUR_WIN].right,win_data[CUR_WIN].top,
              win_data[CUR_WIN].right,
              win_data[CUR_WIN].bottom, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(ltcorner,win_data[CUR_WIN].left,win_data[CUR_WIN].top,
              win_data[CUR_WIN].left,
              win_data[CUR_WIN].top, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(rtcorner,win_data[CUR_WIN].right,win_data[CUR_WIN].top,
              win_data[CUR_WIN].right,
              win_data[CUR_WIN].top, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(lbcorner,win_data[CUR_WIN].left,win_data[CUR_WIN].bottom,
              win_data[CUR_WIN].left,
              win_data[CUR_WIN].bottom, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);
     FillBox(rbcorner,win_data[CUR_WIN].right,win_data[CUR_WIN].bottom,
              win_data[CUR_WIN].right,
              win_data[CUR_WIN].bottom, win_data[CUR_WIN].border_fore_color,
              win_data[CUR_WIN].border_back_color,
              win_data[CUR_WIN].bright_attrib,
              win_data[CUR_WIN].blink_attrib);

////// WRITE THE TITLE ON THE WINDOW //////
     if(win_data[CUR_WIN].bright_attrib==1)
          textcolor(win_data[CUR_WIN].border_fore_color+8);
     else
          textcolor(win_data[CUR_WIN].border_fore_color);
     textbackground(win_data[CUR_WIN].border_back_color);
     if(strcmp(win_data[CUR_WIN].win_title,"")!=0)
     {
          gotoxy(((win_data[CUR_WIN].right+win_data[CUR_WIN].left)/2)-
                  (length/2),win_data[CUR_WIN].top);
          cprintf("%s",win_data[CUR_WIN].win_title);
     }
     textcolor(win_data[CUR_WIN].text_color);
     textbackground(win_data[CUR_WIN].text_back_color);

////// CLEAR THE WINDOW SCREEN /////
     window(win_data[CUR_WIN].left+1,
            win_data[CUR_WIN].top+1,win_data[CUR_WIN].right-1,
            win_data[CUR_WIN].bottom-1);
     clrscr();
     return(0);
}

// CLOSE A WINDOW //
// This function will close the active window

int CloseWindow()
{
     window(1,1,80,25);
     if(CUR_WIN>0)
     {
          puttext(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
                  win_data[CUR_WIN].right+1,win_data[CUR_WIN].bottom+1,
                  win_data[CUR_WIN].under_the_window);
          free(win_data[CUR_WIN].under_the_window);
          CUR_WIN--;
          BWindow();
          puttext(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
                  win_data[CUR_WIN].right,win_data[CUR_WIN].bottom,
                  win_data[CUR_WIN].win_contents);
          free(win_data[CUR_WIN].win_contents);
          gotoxy(win_data[CUR_WIN].cursor_x,
                 win_data[CUR_WIN].cursor_y);
     }
     return(0);
}


/* WINDOW MANIPULATION FUNCTIONS ******************************************/
/* These functions will allow you to manipulate the active window by moving,
   sliding, or just general appearance changes    */


int ColorWindowBorder(int border_back,int border_fore,int bright_attrib,int blink_attriber)
{
     get_contents();
     win_data[CUR_WIN].border_back_color=border_back;
     win_data[CUR_WIN].border_fore_color=border_fore;
     win_data[CUR_WIN].bright_attrib=bright_attrib;
     win_data[CUR_WIN].blink_attrib=blink_attriber;
     BWindow();
     replace_contents();
     return(0);
}


int ColorWindowText(int text_back, int text_color)
{
     win_data[CUR_WIN].text_back_color=text_back;
     win_data[CUR_WIN].text_color=text_color;
     BWindow();
     return(0);
}



int ChangeWindowBorder(int win_border_type)
{
     get_contents();
     win_data[CUR_WIN].win_border_type=win_border_type;
     BWindow();
     replace_contents();
     return(0);
}


int WindowShadowOn()
{
     get_contents();
     win_data[CUR_WIN].win_shadow=SHADOW;
     BWindow();
     replace_contents();
     return(0);
}


int WindowShadowOff()
{
     get_contents();
     win_data[CUR_WIN].win_shadow=NO_SHADOW;
     puttext(win_data[CUR_WIN].left,win_data[CUR_WIN].top,
             win_data[CUR_WIN].right+1,win_data[CUR_WIN].bottom+1,
             win_data[CUR_WIN].under_the_window);
     BWindow();
     replace_contents();
     return(0);
}

int ChangeWindowTitle(char *new_title)
{
     get_contents();
     win_data[CUR_WIN].win_title=new_title;
     BWindow();
     replace_contents();
     return(0);
}


int get_contents(void)
{
     int buf;
     buf=(win_data[CUR_WIN].bottom-win_data[CUR_WIN].top)*
          (win_data[CUR_WIN].right-win_data[CUR_WIN].left)*2;
     win_data[CUR_WIN].win_contents=(int *)malloc(buf);
     gettext(win_data[CUR_WIN].left+1,win_data[CUR_WIN].top+1,
             win_data[CUR_WIN].right-1,win_data[CUR_WIN].bottom-1,
             win_data[CUR_WIN].win_contents);
     win_data[CUR_WIN].cursor_x=wherex();
     win_data[CUR_WIN].cursor_y=wherey();
     return(0);
}

int replace_contents(void)
{
     window(1,1,80,25);
     puttext(win_data[CUR_WIN].left+1,win_data[CUR_WIN].top+1,
             win_data[CUR_WIN].right-1,win_data[CUR_WIN].bottom-1,
             win_data[CUR_WIN].win_contents);
     free(win_data[CUR_WIN].win_contents);
     window(win_data[CUR_WIN].left+1,win_data[CUR_WIN].top+1,
            win_data[CUR_WIN].right-1,win_data[CUR_WIN].bottom-1);
     gotoxy(win_data[CUR_WIN].cursor_x,
            win_data[CUR_WIN].cursor_y);

     return(0);
}


int ClearWindow()
{
     BWindow();
     return(0);
}




int MoveWindow(int top, int left)
{
     int temp_right,temp_bottom;
     struct win_data_ovl temp_window;
     get_contents();
     temp_window.top=win_data[CUR_WIN].top;
     temp_window.bottom=win_data[CUR_WIN].bottom;
     temp_window.left=win_data[CUR_WIN].left;
     temp_window.right=win_data[CUR_WIN].right;
     temp_window.border_back_color=win_data[CUR_WIN].border_back_color;
     temp_window.border_fore_color=win_data[CUR_WIN].border_fore_color;
     temp_window.bright_attrib=win_data[CUR_WIN].bright_attrib;
     temp_window.blink_attrib=win_data[CUR_WIN].blink_attrib;
     temp_window.text_back_color=win_data[CUR_WIN].text_back_color;
     temp_window.text_color=win_data[CUR_WIN].text_color;
     temp_window.win_title=win_data[CUR_WIN].win_title;
     temp_window.win_shadow=win_data[CUR_WIN].win_shadow;
     temp_window.win_border_type=win_data[CUR_WIN].win_border_type;
     temp_window.under_the_window=win_data[CUR_WIN].under_the_window;
     temp_right=temp_window.right-temp_window.left;
     temp_bottom=temp_window.bottom-temp_window.top;
     if(left+temp_right>79)
          return(0);
     if(top+temp_bottom>24)
          return(0);
     temp_window.left=left;
     temp_window.right=temp_window.left+temp_right;
     temp_window.top=top;
     temp_window.bottom=temp_window.top+temp_bottom;
     CloseWindow();
     MakeWindow(temp_window.left,temp_window.top,
                 temp_window.right,temp_window.bottom,
                 temp_window.border_back_color,temp_window.border_fore_color,
                 temp_window.bright_attrib,temp_window.blink_attrib,
                 temp_window.text_back_color,temp_window.text_color,
                 temp_window.win_title,temp_window.win_shadow,
                 temp_window.win_border_type);
     replace_contents();
     return(0);
}
// STRING MANIPULATION FUNCTIONS //
// This function will allow you to get a string input from the user with
// limited editing capabilities at the user end.  You can modify this
// to return the key code for function keys.

int GetStr(char *name_ptr,int size,char marker)
{
   char choice;
   int max_string_size=0;
   int cursor_position=0,x,y,where_x,where_y;
   char first_time='y';
   max_string_size=strlen(name_ptr);
   cprintf("%s",name_ptr);
   for(y=max_string_size;y<size-1;y++)
      cprintf("%c",marker);
   for(y=cursor_position;y<size-1;y++)
      cprintf("\x8");
   while(TRUE)
   {
      if((choice=getch())==0)
      {
         choice=getch();
         switch(choice)
         {
            case 83:if(cursor_position <max_string_size)
                    {
                         for(x=cursor_position;x<max_string_size;x++)
                         name_ptr[x]=name_ptr[x+1];
                         max_string_size--;
                         name_ptr[x-1]=marker;
                         for(x=cursor_position;x<max_string_size+1;x++)
                              cprintf("%c",name_ptr[x]);
                         for(x=cursor_position;x<max_string_size+1;x++)
                              cprintf("\x8");
                    }
                    else
                         cprintf("\x7");
                         break;
            case 75: if(cursor_position>0)
                     {
                        cprintf("\x8");
                        cursor_position--;
                      }
                      break;
            case 77: if(cursor_position<max_string_size)
                     {
                        if(first_time=='y')
                        {
                           first_time='n';
                         }
                      cursor_right();
                      cursor_position++;
                      }
                      break;
            }
         }
         else
         {
         switch(choice)
         {
            case 13:for(x=max_string_size;x<size;x++)
                          name_ptr[x]='\0';
                    return(0);
            case 8:if((cursor_position<max_string_size)&&(cursor_position>0))
                   {
                      cprintf("\x8");
                      cursor_position--;
                   }
                   else
                      if(cursor_position>0)
                      {
                         cprintf("\x8");
                         cprintf("%c",marker);
                         cprintf("\x8");
                         cursor_position--;
                         max_string_size--;
                      }
                   break;
            case 27:STRING_ABORT=1;
                    for(x=0;x<size;x++)
                    name_ptr[x]='';
                    return(0);
            default:if(cursor_position>size-2)
                       cprintf("\x7");
                    else
                    {
                       cprintf("%c",choice);
                       name_ptr[cursor_position]=choice;
                       cursor_position++;
                       if(cursor_position>=max_string_size)
                       max_string_size=cursor_position;
                    }
                    break;
         }
      }
   }
   return(0);
}

int cursor_right()
/*  This function will move the cursor to the right with out the use of
    ansi characters                                                  */
{
   int xposition, yposition;
   xposition=wherex();
   yposition=wherey();
   xposition++;
   gotoxy(xposition,yposition);
   return(0);
}


/*   This function will allow you to place text in the current window at
     the current cursor location in a color different from the default
     colors.  The function will then return the attributes to the default
     colors.
*/
int WText(char *new_text, int new_text_color, int new_back_color)
{
     textcolor(new_text_color);
     textbackground(new_back_color);
     cprintf(new_text);
     textcolor(win_data[CUR_WIN].text_color);
     textbackground(win_data[CUR_WIN].text_back_color);
     return(0);
}


/*   This function works the same as the GetStr() function except that
     it will allow you to change the color of the string that you are
     receiving.
*/
int CGetStr(char *new_text, int size, char filler,int new_text_color,
        int new_back_color)
{
     textcolor(new_text_color);
     textbackground(new_back_color);
     GetStr(new_text,size,filler);
     textcolor(win_data[CUR_WIN].text_color);
     textbackground(win_data[CUR_WIN].text_back_color);
     return(0);
}


int GetAt(int x, int y, char *string, int size, char filler)
{
     get_list[get_counter].x=x;
     get_list[get_counter].y=y;
     get_list[get_counter].string=string;
     get_list[get_counter].size = size;
     get_list[get_counter].filler=filler;
     gotoxy(x,y);
     cprintf("%s",get_list[get_counter].string);
     get_counter++;
     max_get_counter = get_counter;
     return(0);
}


int ReadGets()
{
     new_counter=0;
     while(new_counter < get_counter)
     {
          gotoxy(get_list[new_counter].x,get_list[new_counter].y);
          GetGetStr(get_list[new_counter].string, get_list[new_counter].size,
                 get_list[new_counter].filler);
          new_counter++;
     }
     get_counter=0;
     max_get_counter=0;
     return(0);
}


int GetGetStr(char *name_ptr,int size,char marker)
{
   char choice;
   int max_string_size=0;
   int cursor_position=0,x,y,where_x,where_y;
   char first_time='y';
   max_string_size=strlen(name_ptr);
   cprintf("%s",name_ptr);
   for(y=max_string_size;y<size-1;y++)
      cprintf("%c",marker);
   for(y=cursor_position;y<size-1;y++)
      cprintf("\x8");
   while(TRUE)
   {
      if((choice=getch())==0)
      {
         choice=getch();
         switch(choice)
         {
            case 72:if(new_counter>0)
                         new_counter-=2;
                    return(0);
            case 80:if(new_counter<max_get_counter+1)
                         return(0);
            case 83:if(cursor_position <max_string_size)
                    {
                         for(x=cursor_position;x<max_string_size;x++)
                         name_ptr[x]=name_ptr[x+1];
                         max_string_size--;
                         name_ptr[x-1]=marker;
                         for(x=cursor_position;x<max_string_size+1;x++)
                              cprintf("%c",name_ptr[x]);
                         for(x=cursor_position;x<max_string_size+1;x++)
                              cprintf("\x8");
                    }
                    else
                         cprintf("\x7");
                         break;
            case 75: if(cursor_position>0)
                     {
                        cprintf("\x8");
                        cursor_position--;
                      }
                      break;
            case 77: if(cursor_position<max_string_size)
                     {
                        if(first_time=='y')
                        {
                           first_time='n';
                         }
                      cursor_right();
                      cursor_position++;
                      }
                      break;
            }
         }
         else
         {
         switch(choice)
         {
            case 13:for(x=max_string_size;x<size;x++)
                          name_ptr[x]='\0';
                    return(0);
            case 8:if((cursor_position<max_string_size)&&(cursor_position>0))
                   {
                      cprintf("\x8");
                      cursor_position--;
                   }
                   else
                      if(cursor_position>0)
                      {
                         cprintf("\x8");
                         cprintf("%c",marker);
                         cprintf("\x8");
                         cursor_position--;
                         max_string_size--;
                      }
                   break;
            case 27:for(x=max_string_size;x<size;x++)
                         name_ptr[x]='\0';
                    return(0);
            default:if(cursor_position>size-2)
                       cprintf("\x7");
                    else
                    {
                       cprintf("%c",choice);
                       name_ptr[cursor_position]=choice;
                       cursor_position++;
                       if(cursor_position>=max_string_size)
                       max_string_size=cursor_position;
                    }
                    break;
         }
      }
   }
   return(0);
}

